\name{maop}
\alias{maop}
\alias{print.maop}
\alias{summary.maop}
\title{Mixed Autoregressive Ordered Probit Models}
\description{
Fits mixed autoregressive ordered probit models to a longitudinal data set by maximum pairwise likelihood.
}
\usage{
maop(formula, subjects, periods=subjects, randomEffect=FALSE, 
dynamics=NULL, startValues=NULL, distance=2, print.info=TRUE,
RBR=FALSE, reparam=TRUE, approx_hessian=FALSE, methods=c("Nelder-Mead", "BFGS"), 
all.methods=FALSE, grad.tol.conv=1e-2,...)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{formula}{a formula expression of form
  \code{response ~ predictors}. The response must be an ordinal factor.  The model fitted is a dynamic generalization of the ordered probit
  regression model and it must have an intercept. See the
  documentation of \code{\link{formula}} for other details.
}
\item{subjects}{identifier for the different subjects in the
  longitudinal study. This is a vector of the same length of the response containing
    identifiers for the different subjects. See the
    example.}
\item{periods}{identifier for the different measurement periods of the subjects in the longitudinal study. The
    default value is equal to that of \code{subjects} assuming one single measurement period for each patient. Otherwise, \code{periods} is a vector of the same length of the response containing
    identifiers for the different measurement periods. See the
    example. } 
\item{randomEffect}{logical. Should a subject-specific random effect be
    included?}
\item{dynamics}{a factor whose levels correspond to different
    autocorrelation parameters. The default value is \code{NULL}
    indicating an unique autocorrelation parameter.}
\item{startValues}{starting values. The default value is \code{NULL}
  meaning that starting values are computed automatically by the
  \code{maop} function}  
\item{distance}{maximal distance within pairs of observations included in the pseudolikelihood.}
\item{print.info}{logical. Should information about computation
    status be printed?.}
\item{RBR}{logical. Should the penalized pairwise log-likelihood be compiuted?.}
\item{reparam}{logical. Should the parametrization in Kosmidis and Lunardon (2020) by used in place of the parametrization used by Varin and Czado (2010)?}
\item{approx_hessian}{logical. Should the approximation of the hessian used by Varin and Czado (2010) used to compute standard errors?}
\item{methods}{character. A vector containing a list of methods to be used by optimx to optimize the objective function.}
\item{all.methods}{logical. Should optimx try all the available methods implemented? If set to TRUE this option overrides the argument methods.}
\item{grad.tol.conv}{numeric. Convergence is supposed to be achieved if the largest component of the gradient at the solution is, in absolute value, less than the specified value.}
\item{...}{Further arguments to be passed to function optimx.}
}
\details{\acronym{MAOP} stands for
  \emph{mixed autoregressive ordered probit} models, as described in
  Varin and Czado (2010). Model fitting is performed by maximum pairwise
  likelihood (Varin, 2008).}

\value{
  An object of class \code{maop}. The
  components of an object of the class \code{maop} are

  \item{formula}{fitted model.}
  \item{subjects}{factor denoting different subjects.}
  \item{periods}{factor denoting different measurement periods.}
  \item{isRandomEffect}{whether a subject-specific random effect is
    included in the model.}
  \item{dynamics}{factor denoting different autocorrelation
    parameters.}
  \item{distance}{maximal admissible distance between pairs of observations.}
  \item{startValues}{starting values.}
  \item{coeff}{maximum pairwise likelihood estimator.}
  \item{stdErr}{estimated standard errors.}
  \item{varMatrix}{estimated variance-covariance matrix of the parameter estimates.}
  \item{logPair}{maximized log-pairwise likelihood.}
  \item{clic}{composite likelihood information criterion (Varin and  Vidoni, 2005).}
  \item{convergence}{integer code that takes value 1 if the criterion described in \code{grad.tol.conv} is reached, otherwise 0. In the latter case the function provides \code{NA} for the quantities \code{coeff}, \code{stdErr}, \code{varMatrix}, \code{logPair}, and \code{clic}; the user can gather some details by inspecting the output provided in \code{opt.pair}.}
  \item{opt.pair}{the full output returned by function optimx.}
}
\references{
Varin, C. and Vidoni, P. (2005). A note on composite likelihood
inference and model selection. \emph{Biometrika}, \bold{92}, 519--528.

Varin, C. (2008). On composite marginal likelihoods. \emph{Advances in
  Statistical Analysis}, \bold{92}, 1--28.

Varin, C. and Czado, C. (2010). A mixed autoregressive probit model for
  ordinal longitudinal data. \emph{Biostatistics}, \bold{11}, 127--138. 
  }
\author{Cristiano Varin}
\seealso{\code{\link{migraine}}, \code{\link{optim}}, \code{\link[MASS]{polr}}}

\examples{
# Fits a MAOP model to migraine data (Varin and Czado, 2009)
data(migraine)
attach(migraine)
# covariates
university <-rep(0,length(HA))
university[ED==5]<-1
university[ED==6]<-1
university[ED==7]<-1
university <- as.factor(university==1)
analgesics <- as.factor(MEDANALG=="Y")
# weather covariates
wc <- factor(cut(MINWC, breaks=c(-50, -10, 0, 10, 30)))
high.pressure <- PMN>10130
high.pressure.yesterday <- PMN1P>10130
change <- rep("persistence", length(high.pressure))
change[(!high.pressure & high.pressure.yesterday)] <- "worse"
change[(high.pressure & !high.pressure.yesterday)] <- "better"
change <- factor(change)
humidity <- cut(RHMN, breaks=c(0, 60, 80, 100))
# base model
m <- HA~university+analgesics
# fit model m with different autocorrelation parameters 
# for patients with analgesics intake and those who not 
# try with pairwise likelihood with maximal distance 6
# note that we are using the parametrization in Kosmidis and Lunardon (2020)
maop6 <- maop(m, subjects=SUBJECT, periods=PERIOD, randomEffect=TRUE, 
dynamics=analgesics, distance=6)
summary(maop6, digits=2)
# try with penalized pairwise likelihood
maop6.pen <- maop(m, subjects=SUBJECT, periods=PERIOD, randomEffect=TRUE, 
dynamics=analgesics, distance=6, RBR=TRUE, startValues=maop6$coeff)
summary(maop6.pen, digits=2)

}
\keyword{Longitudinal data, ordinal categorical response, pairwise
  likelihood, random effects.}
