## Copyright (C) 2020 Ioannis Kosmidis

## Distributed as part of the supplementary material for the manuscript
## "Empirical bias-reducing adjustments to estimating functions"
##
## Authors: Ioannis Kosmidis
## Date: 06 January 2020
## Licence: GPL 2 or greater
## NOT A POLISHED PIECE OF PUBLIC-USE SOFTWARE!  Provided "as is".
## NO WARRANTY OF FITNESS FOR ANY PURPOSE!

library("parallel")
library("MASS")
library("numDeriv")
library("data.table")

source("brquasiControl.R")
source("brquasi.R")
source("negbin_functions.R")

## save path
path <- "."

## random number generatoe set to L'Ecuyer-CMRG for use with mclapply
RNGkind("L'Ecuyer-CMRG")

## number of rows of base X matrix
base_n <- 20

## number of base simulations
base_n_simu <- 200

## base model matrix
set.seed(123)
base_x <- cbind(1, rbinom(base_n, 1, 0.5), rexp(base_n, 2))

## parameters
beta <- c(2, 1, -1)
psi <- 1/5
truth <- c(beta, psi2phi(psi))

multiplier <- 2^(0:7)
n_obs <- base_n * multiplier
n_simu <- base_n_simu * n_obs

## number of cores; keep it as is for full reproducibility
n_cores <- c(rep(16, 7), 8)


replications <- 5
seeds <- matrix(round(runif(length(n_obs) * replications, 0, 10^6)), length(n_simu), replications)

## Alternatives with data generated on the fly

## Variance functions is phi * mu while generating process is phi * mu
repl <- 1:replications
for (j in seq_along(n_simu)) {
    ## Current model matrix
    X <- base_x[rep(seq.int(base_n), multiplier[j]), ]
    for (r in repl) {
        set.seed(seeds[j, r])
        results <- mclapply(seq.int(n_simu[j]), function(i) {
            if (i %% 1000 == 0) {
                cat("n =", n_obs[j], "| replication", r, "| sample", i, "/", n_simu[j], "\n")
            }
            y <- simulate_nb(truth, X)
            out <- fit1_br(y = y, x = X, alt_start = truth)
            out$sample <- i
            out
        },
        mc.cores = n_cores[j],
        mc.set.seed = TRUE)
        results <- as.data.frame(rbindlist(results))##do.call("rbind", res)
        results$true_value <- truth
        attr(results, "replication") <- r
        attr(results, "N") <- n_simu[j]
        attr(results, "n") <- n_obs[j]
        save(list = "results",
             file = paste0(path, "/brquasi-mu-negbin-", j, "-repl-", r, ".rda"))
    }
}


## Variance functions is phi * mu^2 while generating process is phi*mu
repl <- 1:replications
for (j in seq_along(n_simu)) {
    ## Current model matrix
    X <- base_x[rep(seq.int(base_n), multiplier[j]), ]
    for (r in repl) {
        set.seed(seeds[j, r])
        results <- mclapply(seq.int(n_simu[j]), function(i) {
            if (i %% 1000 == 0) {
                cat("n =", n_obs[j], "| replication", r, "| sample", i, "/", n_simu[j], "\n")
            }
            y <- simulate_nb(truth, X)
            out <- fit2_br(y = y, x = X, alt_start = truth)
            out$sample <- i
            out
        },
        mc.cores = n_cores[j],
        mc.set.seed = TRUE)
        results <- as.data.frame(rbindlist(results))##do.call("rbind", res)
        results$true_value <- truth
        attr(results, "replication") <- r
        attr(results, "N") <- n_simu[j]
        attr(results, "n") <- n_obs[j]
        save(list = "results",
             file = paste0(path, "/brquasi-musquared-negbin-", j, "-repl-", r, ".rda"))
    }
}




## ## This function is only useful for traceback when V(mu) = phi * mu
## reproduce1 <- function(j, r, i) {
##     X <- base_x[rep(seq.int(base_n), multiplier[j]), ]
##     set.seed(seeds[j, r])
##     results <- mclapply(seq.int(n_simu[j]), function(k) {
##         y <- simulate_nb(truth, X)
##         if (k == i) {
##             return(fit1_br(y = y, x = X, alt_start = truth))
##         }
##         else {
##             return(NULL)
##         }
##     },
##     mc.cores = n_cores,
##     mc.set.seed = TRUE)
##     results[[i]]
## }


## ## This function is only useful for traceback when V(mu) = phi * mu
## reproduce2 <- function(j, r, i) {
##     X <- base_x[rep(seq.int(base_n), multiplier[j]), ]
##     set.seed(seeds[j, r])
##     results <- mclapply(seq.int(n_simu[j]), function(k) {
##         y <- simulate_nb(truth, X)
##         if (k == i) {
##             return(list(res = fit2_br(y = y, x = X, alt_start = truth),
##                    y = y,
##                    x = X))
##         }
##         else {
##             return(NULL)
##         }
##     },
##     mc.cores = n_cores[j],
##     mc.set.seed = TRUE)
##     results[[i]]
## }
##
## ## Variance functions is phi * mu while generating process is phi * mu
## repl <- 1:replications
## for (j in seq_along(n_simu)) {
##     ## Current model matrix
##     X <- base_x[rep(seq.int(base_n), multiplier[j]), ]
##     for (r in repl) {
##         set.seed(seeds[j, r])
##         simu_data <- replicate(n_simu[j], simulate_nb(truth, X))
##         results <- mclapply(seq.int(n_simu[j]), function(i) {
##             if (i %% 100 == 0) {
##                 cat("n =", n_obs[j], "| replication", r, "| sample", i, "/", n_simu[j], "\n")
##             }
##             out <- fit1_br(y = simu_data[, i], x = X, alt_start = truth)
##             out$sample <- i
##             out
##         }, mc.cores = n_cores[j])
##         results <- as.data.frame(rbindlist(results))##do.call("rbind", res)
##         results$true_value <- truth
##         attr(results, "replication") <- r
##         attr(results, "N") <- n_simu[j]
##         attr(results, "n") <- n_obs[j]
##         save(list = "results",
##              file = paste0(path, "/brquasi-mu-negbin-", j, "-repl-", r, ".rda"))
##     }
## }
## ## Variance functions is phi * mu^2 while generating process is phi*mu
## repl <- 1:replications
## for (j in seq_along(n_simu)) {
##     ## Current model matrix
##     X <- base_x[rep(seq.int(base_n), multiplier[j]), ]
##     for (r in repl) {
##         set.seed(seeds[j, r])
##         simu_data <- replicate(n_simu[j], simulate_nb(truth, X))
##         results <- mclapply(seq.int(n_simu[j]), function(i) {
##             if (i %% 100 == 0) {
##                 cat("n =", n_obs[j], "| replication", r, "| sample", i, "/", n_simu[j], "\n")
##             }
##             out <- fit2_br(y = simu_data[, i], x = X, alt_start = truth)
##             out$sample <- i
##             out
##         }, mc.cores = n_cores[j])
##         results <- as.data.frame(rbindlist(results))##do.call("rbind", res)
##         results$true_value <- truth
##         attr(results, "replication") <- r
##         attr(results, "N") <- n_simu[j]
##         attr(results, "n") <- n_obs[j]
##         save(list = "results",
##              file = paste0(path, "/brquasi-musquared-negbin-", j, "-repl-", r, ".rda"))
##     }
## }
