## Copyright (C) 2020 Ioannis Kosmidis

## Distributed as part of the supplementary material for the manuscript
## "Empirical bias-reducing adjustments to estimating functions"
##
## Authors: Ioannis Kosmidis
## Date: 06 January 2020
## Licence: GPL 2 or greater
## NOT A POLISHED PIECE OF PUBLIC-USE SOFTWARE!  Provided "as is".
## NO WARRANTY OF FITNESS FOR ANY PURPOSE!

## Ratio estimator from Example 2 in Stefanski & Boos (2002)

## Data
library("mvtnorm")
library("parallel")
library("data.table")
library("copula")

## path to save intermediate results
path <- "./images/"

## number of cores
n_cores <- 2

## number of rows of base X matrix
base_n <- 5

## number of base simulations
base_n_simu <- 250 * 2^16 ## 2000

## n_obs and n_simu
multiplier <- 2^(0:6)
n_obs <- base_n * multiplier
n_simu <- base_n_simu / n_obs

## Parameter setting
setting <- list(sigma = matrix(c(1, 0.5, 0.5, 1), ncol = 2),
                mu = c(2, 10))

sigma <- setting$sigma
mu <- setting$mu
theta <- mu[2]/mu[1]

set.seed(123)
seeds <- round(runif(length(n_obs), 0, 10^6))

simu_fun <- function(nsimu, mu, sigma) {
    out <- rCopula(n = nsimu, normalCopula(sigma[1,2], 2))
    t(apply(out, 1, function(x) c(qexp(x[1], 1/mu[1]), qnorm(x[2], mu[2], sqrt(sigma[2, 2])))))
}

estimate_ratio <- function(D, truth) {
    sx <- sum(D[, 1])
    sy <- sum(D[, 2])
    sxy <- sum(D[, 1] * D[, 2])
    sxx <- sum(D[, 1] * D[, 1])
    syy <- sum(D[, 2] * D[, 2])
    n <- nrow(D)
    mle <- sy / sx
    br <- (sy * sx + sxy) / (sx^2 + sxx)
    bc <- mle * (1 - sxx / sx^2) + sxy/ sx^2
    mle_i <- sapply(1:n, function(i) { m <- colSums(D[-i, ]); m[2]/m[1] })
    jack <- n_obs[j] * mle - (n_obs[j] - 1)/n_obs[j] * sum(mle_i)
    e <- function(theta) (theta^2 * sxx + syy - 2 * theta * sxy)
    v <- function(theta) e(theta)/sx^2
    sbr <- function(theta) sy - theta * sx + sxy/sx - theta * sxx/sx
    s <- function(theta) sy - theta * sx       
    br_var <- v(br)
    bc_var <- v(bc)
    mle_var <- v(mle)
    jack_var <- sum((mle_i - mean(mle_i))^2) * (n - 1) / n   
    br_pvalue_e <- 1 - pchisq((br - truth)^2/br_var, 1)
    bc_pvalue_e <- 1 - pchisq((bc - truth)^2/bc_var, 1)
    mle_pvalue_e <- 1 - pchisq((mle - truth)^2/mle_var, 1)
    mle_pvalue_s <- 1 - pchisq(s(truth)^2 / e(truth), 1)
    br_pvalue_s <- 1 - pchisq(sbr(truth)^2 / e(truth), 1)
    k <- attr(D, "sample")
    if (k %% 1000 == 0) {
        cat("n: ", n, "size:", n_simu[j], "|",  "sample:", k, "Done!\n")
    }
    data.frame(parameter = c("theta", "theta", "theta", "theta"),
               method = c("ML", "BR", "Jackknife", "BC"),
               estimate = c(mle, br, jack, bc),
               evar = c(mle_var, br_var, jack_var, bc_var),
               pvalue_e = c(mle_pvalue_e, br_pvalue_e, NA, bc_pvalue_e),
               pvalue_s = c(mle_pvalue_s, br_pvalue_s, NA, NA),
               sample = k)
}

for (j in 2:length(n_simu)) {
    ## Simulate data
    set.seed(seeds[j])
    simu_data <- data.frame(simu_fun(n_obs[j] * n_simu[j], mu, sigma))
    names(simu_data) <- c("x", "y")
    simu_data <- split(simu_data, factor(rep(1:n_simu[j], each = n_obs[j])))
    simu_data <- lapply(seq.int(n_simu[j]), function(k) {D <- simu_data[[k]]; attr(D, "sample") <- k; D})
    ## Estimates
    res <- mclapply(simu_data, estimate_ratio , truth = mu[2]/mu[1], mc.cores = n_cores)
    ## Save
    results <- rbindlist(res)
    results$true_value <- theta
    attr(results, "N") <- n_simu[j]
    attr(results, "n") <- n_obs[j]
    save(list = "results",
         file = paste0(path, "/ratio-setting-copula", "-n-", j, ".rda"))              
}
